# __main__.py
"""
Author : Christian Ayala <cayalaortiz@email.arizona.edu>
Date   : 2021-04-10
Purpose: Program to run MetaboDirect scripts

To get help, use metabodirect -h or visit the website https://github.com/Coayala/MetaboDirect
"""

import os
import datetime
import sys
import pandas as pd
import py4cytoscape as p4c
from metabodirect import get_args, preprocessing, diagnostics, r_control, transformations


# --------------------------------------------------
def main():
    """Main body to run all MetaboDirect scripts"""

    args = get_args.get_args()

    print('========================\nWelcome to MetaboDirect\n========================\n')
    print('Analysis starting on {}'.format(datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S %p")))
    print('Results will be saved in directory: {}\n'.format(os.path.abspath(args.outdir)))

    print(f'Data file is {args.data_file}')
    print(f'Metadata file is {args.metadata_file}')

    print('\n------------------------\nStarting data pre-processing\n------------------------\n')

    df = pd.read_csv(args.data_file)
    metadata = pd.read_csv(args.metadata_file)
    list_dir = preprocessing.make_directories(args.outdir)

    if args.filter_by:
        print(f'Filtering samples based on {args.filter_by[0]} = {args.filter_by[1]}')
        df, metadata = preprocessing.sample_filtering(df, metadata, args.filter_by, path=list_dir[0])
        print(f'Filtered tables can be found in the directory: {os.path.abspath(list_dir[0])}')
    else:
        print('Option -f not detected, all samples will be used')

    df = preprocessing.data_filtering(df, args.mass_filter)
    df = preprocessing.thermo_idx_and_classes(df)
    df = preprocessing.data_normalization(df, args.norm_method, args.norm_subset, args.subset_parameter,
                                          args.log_transform)

    df_formulas = df[df['C'] > 0]

    filename = os.path.join(list_dir[0], 'Report_processed.csv')
    df.to_csv(filename, index=False)
    filename = os.path.join(list_dir[0], 'Report_processed_MolecFormulas.csv')
    df_formulas.to_csv(filename, index=False)
    print(f'Report saved as: {filename}')

    preprocessing.calculate_summaries(df_formulas, path=list_dir[0])
    matrix_features = preprocessing.get_matrix(df_formulas)
    matrix_features.to_csv(os.path.join(list_dir[0], 'matrix_features.csv'))

    colnames = [col for col in list(df_formulas.columns) if col not in list(metadata['SampleID'])]
    df = df_formulas.melt(id_vars=colnames,
                          var_name='SampleID', value_name='NormIntensity')
    df = df[df['NormIntensity'] > 0].reset_index(drop=True)
    df = df.merge(metadata, on='SampleID')

    print('\n------------------------\nData pre-processing finished\n------------------------\n')

    print('------------------------\nStarting data diagnostics\n------------------------\n')

    print('Calculating number of assigned molecular formulas per sample')
    diagnostics.formula_per_sample(df, metadata, path=list_dir[1])
    print("Calculating error distribution per group/s: {}".format(args.group))
    diagnostics.error_per_group(df, args.group, path=list_dir[1])

    print('\n------------------------\nData diagnostics finished\n------------------------\n')

    print('------------------------\nStarting data exploration\n------------------------\n')

    data_exploration_script = r_control.write_r_script('data_exploration_template.R', outdir=list_dir[2],
                                                       metadata_file=args.metadata_file if not args.filter_by
                                                       else os.path.join(list_dir[0], 'filtered_metadata.csv'),
                                                       groups=args.group)
    print(f'Running R script: {data_exploration_script}')
    r_control.run_r(data_exploration_script)
    print(f'Find results and R script in the directory: {os.path.abspath(list_dir[2])}')

    if args.kegg_annotation:
        print('Starting annotation of molecular formulas using the KEGG database')
        kegg_annotation_script = r_control.write_r_script('KEGG_annotation_template.R', outdir=list_dir[2],
                                                          metadata_file=args.metadata_file if not args.filter_by
                                                          else os.path.join(list_dir[0], 'filtered_metadata.csv'),
                                                          groups=args.group)
        r_control.run_r(kegg_annotation_script)
        print(f'\nFind results and R script in the directory: {os.path.abspath(list_dir[2])}')
    else:
        print('\nKEGG annotation not selected. If you wish to perform a KEGG annotation run the script again using'
              'the -k/--kegg_annotation option')

    print('\n------------------------\nData Exploration finished\n------------------------\n')

    print('------------------------\nStarting statistical analysis\n------------------------\n')

    data_statistics_script = r_control.write_r_script('data_statistics_template.R', outdir=list_dir[3],
                                                      metadata_file=args.metadata_file if not args.filter_by
                                                      else os.path.join(list_dir[0], 'filtered_metadata.csv'),
                                                      groups=args.group, norm_method=args.norm_method)
    print(f'Running R script: {os.path.abspath(data_statistics_script)}')
    r_control.run_r(data_statistics_script)
    print(f'Find results and R script in the directory: {os.path.abspath(list_dir[3])}')

    print('------------------------\nStarting transformation network analysis\n------------------------\n')

    if not args.calculate_transformations:
        print('Calculate transformations not selected. '
              'If you wish to do calculate transformations based on biochemical key please set the option "-t"')
        sys.exit()

    keys = transformations.get_keys(os.path.join(os.path.split(os.path.realpath(__file__))[0],
                                                 'data',
                                                 'transf_key.csv')) if args.biochem_key == 'Default key' else \
        transformations.get_keys(args.biochem_key)
    transformations.calculate_transformations(df, keys, path=list_dir[5])
    transformations.summarize_transformations(path=list_dir[4])
    node_table = transformations.get_node_table(df, path=list_dir[4])

    print(f'Finished to calculate transformatios, please find transformation files in the directory:'
          f'{os.path.abspath(list_dir[5])}')

    if not args.create_networks:
        print(f'Create networks not selected.'
              f'If you wish to create networks automatically please set the option -c.'
              f'Otherwise to create networks from previously calculated transformations use the following command '
              f'from a terminal window:\n'
              f'> create_networks {args.outdir}')
        sys.exit()

    check = ''
    while check != 'You are connected to Cytoscape!':
        cytoscape = input(f'Please open Cytoscape and press the ENTER key [q for quit].')
        if cytoscape == '':
            try:
                check = p4c.cytoscape_ping()
            except (ValueError, Exception):
                print('Cytoscape is not open.')
        elif cytoscape == 'q':
            sys.exit()
        else:
            check = 'Cytoscape not open'

    print('Starting network construction on Cytoscape')

    transformations.create_cytoscape_network(node_table, path=list_dir[4])

    network_stats_script = r_control.write_r_script('network_stats_template.R', outdir=list_dir[4],
                                                    metadata_file=args.metadata_file if not args.filter_by
                                                    else os.path.join(list_dir[0], 'filtered_metadata.csv'),
                                                    groups=args.group, norm_method=args.norm_method)

    print(f'Running R script: {network_stats_script}')
    r_control.run_r(network_stats_script)
    print(f'Find results and R script in the directory: {os.path.abspath(list_dir[4])}')

    print('------------------------\nTransformation network analysis finished\n------------------------\n')
    print('========================\nThanks for using MetaboDirect\n========================\n')


# --------------------------------------------------
if __name__ == '__main__':
    main()



#!/usr/bin/env python3
"""
Author : Christian Ayala <cayalaortiz@email.arizona.edu>
Date   : 2021-04-10
Purpose: Program to run MetaboDirect scripts
"""

import argparse
import os
import datetime
import subprocess
import pandas as pd
from shutil import which
from platform import system


# --------------------------------------------------
def get_args():
    """Get command-line arguments"""

    parser = argparse.ArgumentParser(
        description='Program for running all the MetaboDirect analysis pipeline',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    parser.add_argument('data_file',
                        help='Name of the file with the DI-MS data in .csv format',
                        metavar='DATA',
                        type=str)

    parser.add_argument('metadata_file',
                        help='Name of the file with the sample information (metadata) in tabular format',
                        metavar='METADATA',
                        type=str)

    parser.add_argument('group',
                        help='Grouping variables to test for normalization significance',
                        metavar='GROUP',
                        type=str)

    parser.add_argument('-f',
                        '--filter_by',
                        help='Filter samples based on metadata. First enter the name of the feature,'
                             'followed by the values associated with the samples you want to keep in the analysis.'
                             '(Example -f Habitat Bog,Palsa)',
                        metavar='STR',
                        type=str,
                        nargs=2)

    parser.add_argument('--log_transform',
                        help='Set this if you plan to log transform your data before normalization',
                        action='store_true',
                        default=False)

    args = parser.parse_args()

    if not os.path.isfile(args.data_file):
        parser.error(f'File {args.data_file} does not exist')

    if not os.path.isfile(args.metadata_file):
        parser.error(f'File {args.metadata_file} does not exist')

    return args


# --------------------------------------------------
def sample_filtering(df, metadata, filter_by):
    """Filter samples based on selected features and values."""

    # Get the variable a values specified for sample filtering
    filter_col = filter_by[0]
    filter_values = filter_by[1].split(sep=',')

    # Saving a new metadata file containing only the samples remaining after filtering
    filt_metadata = pd.DataFrame()
    for i in filter_values:
        filt_metadata = filt_metadata.append(metadata[metadata[filter_col] == i])
    filt_metadata_file = 'filtered_metadata_norm_test.csv'
    filt_metadata.to_csv(filt_metadata_file, index=False)

    # Saving a new input file containing only the samples remaining after filtering
    from_formularity = [
        'Mass', 'C', 'H', 'O', 'N', 'C13', 'S', 'P', 'Na', 'El_comp', 'Class',
        'NeutralMass', 'Error_ppm', 'Candidates'
    ]
    col_df = filt_metadata['SampleID'].to_list()
    from_formularity.extend(col_df)
    filt_df = df[from_formularity]
    filt_df_file = 'filtered_input_norm_test.csv'
    filt_df.to_csv(filt_df_file, index=False)

    return filt_df_file, filt_metadata_file


# --------------------------------------------------
def write_r_script(rscript, data_file, metadata_file, group, log):
    """Function to write R scripts based on the provided Rscripts templates"""
    current_dir = os.getcwd().replace('\\', '/')
    data_file = data_file.replace('\\', '/')
    metadata_file = metadata_file.replace('\\', '/')
    metabo_home = os.path.split(os.path.realpath(__file__))[0].replace('\\', '/')
    r_in = os.path.join(os.path.split(os.path.realpath(__file__))[0], 'R_scripts_templates', rscript)
    r_in = open(r_in)
    r_file = os.path.join(current_dir, rscript.replace('_template', ''))
    r_fh = open(r_file, 'w')
    for line in r_in:
        r_fh.write(line.replace('%currentdir%', current_dir)
                   .replace('%metadata%', metadata_file)
                   .replace('%group%', group)
                   .replace('%data%', data_file)
                   .replace('%Metabo_HOME%', metabo_home)
                   .replace('%log%', 'TRUE' if log else 'FALSE')
                   )
    r_fh.close()

    return r_file


# --------------------------------------------------
def run_r(rscript):
    """Function to execute R scripts"""
    rscript_exe = 'Rscript.exe' if system() == 'Windows' else 'Rscript'

    if which(rscript_exe) is not None:
        r_cmd = [rscript_exe, rscript]
        p = subprocess.Popen(
            r_cmd, shell=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE
        )
        out, err = p.communicate()
        if out:
            print(out)

        if err:
            print(err)
    else:
        print('Rscript not accesible')


# --------------------------------------------------
def main():
    """Main body to run test normalization analysis"""

    args = get_args()

    print('========================\nWelcome to MetaboDirect\n========================\n')
    print('\n------------------------\nNormalization methods testing\n------------------------\n')
    print('Analysis starting on {}'.format(datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S %p")))
    print('Results will be saved in directory: {}\n'.format(os.path.abspath(os.getcwd())))

    print(f'Data file is {args.data_file}')
    print(f'Metadata file is {args.metadata_file}')

    if args.filter_by:
        df = pd.read_csv(args.data_file)
        metadata = pd.read_csv(args.metadata_file)
        print(f'Filtering samples based on {args.filter_by[0]} = {args.filter_by[1]}')
        df_file, metadata_file = sample_filtering(df, metadata, args.filter_by)
        args.data_file = df_file
        args.metadata_file = metadata_file

    else:
        print('Option -f not detected, all samples will be used')

    spans_score_script = write_r_script('SPANS_score_template.R', args.data_file, args.metadata_file,
                                        args.group, args.log_transform)

    print(f'Running R script: {spans_score_script}')
    run_r(spans_score_script)
    print(f'Find results and R script in the directory: {os.path.abspath(os.getcwd())}')


# --------------------------------------------------
if __name__ == '__main__':
    main()



# get_args.py

import argparse
import os
import metabodirect


# --------------------------------------------------
def get_args():
    """Get command-line arguments."""

    parser = argparse.ArgumentParser(
        description='Program for running all the MetaboDirect analysis pipeline',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    parser.add_argument('data_file',
                        help='Name of the file with the Direct Injection MS data in .csv format',
                        metavar='DATA',
                        type=str)

    parser.add_argument('metadata_file',
                        help='Name of the file with the sample information (metadata) in .csv format',
                        metavar='METADATA',
                        type=str)

    parser.add_argument('-o',
                        '--outdir',
                        help='Output directory',
                        metavar='OUTDIR',
                        type=str,
                        default='MetaboDirect_output')

    parser.add_argument('-m',
                        '--mass_filter',
                        help='Range to filter m/z data (min_mz, max_mz). The pipeline will not filter '
                             'm/z values by default',
                        metavar='INT',
                        type=float,
                        nargs=2)

    parser.add_argument('-g',
                        '--group',
                        help='Grouping variables for coloring and faceting figures (Max 2)',
                        metavar='STR',
                        type=str,
                        nargs='+',
                        required=True)

    parser.add_argument('-f',
                        '--filter_by',
                        help='Filter samples based on metadata. First enter the name of the feature,'
                             'followed by the values associated with the samples you want to keep in the analysis.'
                             '(Example -f Habitat Bog,Palsa)',
                        metavar='STR',
                        type=str,
                        nargs=2)

    parser.add_argument('-b',
                        '--biochem_key',
                        help='File with the biochemical key to use for the transformation network',
                        metavar='STR',
                        type=str,
                        default='Default key')

    parser.add_argument('-v',
                        '--version',
                        action='version',
                        version='%(prog)s {version}'.format(version=metabodirect.__version__))

    parser.add_argument('-k',
                        '--kegg_annotation',
                        help='Set this option to perform annotation of the molecular formulas using'
                             'the KEGG database',
                        action='store_true',
                        default=False)

    norm = parser.add_argument_group('Normalization methods',
                                     'Options to define how data normalization will be carried out')

    norm.add_argument('-n',
                      '--norm_method',
                      help="Available methods to normalize data are: 'mean', 'median', 'zscore', "
                           "'sum', 'max', 'minmax', 'binary', 'none'",
                      metavar='STR',
                      type=str,
                      choices=['mean', 'median', 'zscore', 'sum', 'max', 'minmax', 'binary', 'none'],
                      default='max')

    norm.add_argument('--norm_subset',
                      help='Subset of the data to use for normalization purpouses. '
                           'Available subset methods: ALL, LOS, PPP'
                           'LOS uses peaks in the top L '
                           'order statistics, PPP uses peaks having a minimum percentage of observed values.',
                      metavar='STR',
                      type=str,
                      choices=['ALL', 'LOS', 'PPP'],
                      default='ALL'
                      )

    norm.add_argument('--subset_parameter',
                      help='If using a sample subset for nomalization, this parameter defines the subsample of peaks '
                           'that will be used for normalization.'
                           'If not defined, the default values will be 0.3 for LOS and 0.5 for PPP',
                      metavar='FLOAT',
                      type=float
                      )

    norm.add_argument('--log_transform',
                      help='Set this option to log transform the data. (Program will fail if there are peaks with '
                           'intensities of 0. Consider tranforming this values into 1 if log transformation is desired',
                      action='store_true',
                      default=False
                      )

    transf = parser.add_argument_group('Transformation network options',
                                       'Options to control wheter transformations will be calculated and if '
                                       'networks will be constructed')

    transf.add_argument('-t',
                        '--calculate_transformations',
                        help='Set this option to calculate transformations based on biochemical key',
                        action='store_true',
                        default=False)

    transf.add_argument('-c',
                        '--create_networks',
                        help='Set this option to build transformation networks based on transfomations calculated'
                             'with the biochemical key (this options turns -t automatically)',
                        action='store_true',
                        default=False)

    args = parser.parse_args()

    # Check that command line arguments are specified properly

    if not os.path.isfile(args.data_file):
        parser.error(f'File {args.data_file} does not exist')

    if not os.path.isfile(args.metadata_file):
        parser.error(f'File {args.metadata_file} does not exist')

    if len(args.group) > 2:
        parser.error('Incorrect number of variables for grouping. Please enter exactly two values')

    if args.mass_filter:
        if args.mass_filter[1] < args.mass_filter[0]:
            parser.error('Incorrect order of values for filtering. Please enter min_mz first (e.g. -f 200 900)')

    if args.norm_subset != 'ALL':
        if args.subset_parameter > 1:
            parser.error(f'Incorrect subset parameter. Subset parameter must be between 0 and 1 (proportion).')
        if not args.subset_parameter:
            args.subset_parameter = 0.3 if args.norm_subset == 'LOS' else 0.5

    if args.create_networks:
        args.calculate_transformations = True

    return args
